package codechicken.enderstorage;

import java.io.File;

import net.minecraft.command.CommandHandler;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import codechicken.core.CommonUtils;
import codechicken.core.launch.CodeChickenCorePlugin;
import codechicken.lib.config.ConfigFile;
import codechicken.enderstorage.api.EnderStorageManager;
import codechicken.enderstorage.common.BlockEnderStorage;
import codechicken.enderstorage.internal.EnderStorageProxy;
import codechicken.enderstorage.storage.EnderItemStoragePlugin;
import codechicken.enderstorage.storage.EnderLiquidStoragePlugin;
import codechicken.enderstorage.storage.item.CommandEnderStorage;
import codechicken.enderstorage.storage.item.ItemEnderPouch;
import codechicken.packager.Packager;
import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.EventHandler;
import cpw.mods.fml.common.SidedProxy;
import cpw.mods.fml.common.event.FMLInitializationEvent;
import cpw.mods.fml.common.event.FMLPreInitializationEvent;
import cpw.mods.fml.common.event.FMLServerStartingEvent;
import cpw.mods.fml.common.network.NetworkMod;

@Packager(getBaseDirectories = {"EnderStorage"})
@Mod(name="EnderStorage", version="1.4.3.1", useMetadata = false, modid = "EnderStorage", 
    dependencies = "required-after:CodeChickenCore@[0.9.0.0,)", acceptedMinecraftVersions=CodeChickenCorePlugin.mcVersion)
@NetworkMod(clientSideRequired = true, serverSideRequired = true)
public class EnderStorage
{
    @SidedProxy(clientSide="codechicken.enderstorage.internal.EnderStorageClientProxy", serverSide="codechicken.enderstorage.internal.EnderStorageProxy")
    public static EnderStorageProxy proxy;
    
    public static ConfigFile config;
    
    public static BlockEnderStorage blockEnderChest;
    public static ItemEnderPouch itemEnderPouch;
    
    private static int personalItemID;
    public static boolean disableVanillaEnderChest;
    
    @EventHandler
    public void preInit(FMLPreInitializationEvent event)
    {
        config = new ConfigFile(new File(CommonUtils.getMinecraftDir()+"/config", "EnderStorage.cfg")).setComment("EnderStorage Configuration File\nDeleting any element will restore it to it's default value\nBlock ID's will be automatically generated the first time it's run");
        
        proxy.preInit();
    }
    
    @EventHandler
    public void init(FMLInitializationEvent event)
    {
        personalItemID = config.getTag("personalItemID")
                .setComment("The ID of the item used to set the chest to personal. Diamond by default")
                .getIntValue(Item.diamond.itemID);        
        disableVanillaEnderChest = config.getTag("disable-vanilla").setComment("Set to true to make the vanilla enderchest unplaceable.").getBooleanValue(true);
        
        EnderStorageManager.loadConfig(config);
        EnderStorageManager.registerPlugin(new EnderItemStoragePlugin());
        EnderStorageManager.registerPlugin(new EnderLiquidStoragePlugin());
        
        proxy.init();
    }

    @EventHandler
    public void registerCommands(FMLServerStartingEvent event)
    {
        CommandHandler commandManager = (CommandHandler)event.getServer().getCommandManager();
        commandManager.registerCommand(new CommandEnderStorage());
    }
    
    public static ItemStack getPersonalItem()
    {
        if(personalItemID < 0 || personalItemID >= Item.itemsList.length || Item.itemsList[personalItemID] == null)
        {
            System.err.println("EnderStorage: Invalid personal itemID, resetting");
            config.getTag("personalItemID").setIntValue(personalItemID = Item.diamond.itemID);
        }
        
        return new ItemStack(personalItemID, 1, 0);
    }
}
